/*
 * Handles the rewriting of a class to include checks
 */

package rewrite.rewriter;

import rewrite.util.*;

import java.io.*;
import java.util.*;
import java.util.jar.*;

import org.apache.bcel.classfile.*;
import org.apache.bcel.generic.*;
import org.apache.bcel.util.*;
import org.apache.bcel.Constants;

import spox.policy.*;
import spox.policy.pointcut.*;
import spox.policy.PolicyComponent.*;

public class ClassRewriter
{

    // Fields
    private PolicyClass policyClass = null;
    private ClassGen cgen = null;
    private Policy policy = null;
    private HashMap<String, ClassGen> classMap = null;
    
    // Constructor
    public ClassRewriter (
            PolicyClass policyClass,
            ClassGen cgen,
            Policy policy,
            HashMap<String, ClassGen> classMap)
    {
        this.policyClass = policyClass;
        this.cgen = cgen;
        this.policy = policy;
        this.classMap = classMap;
    }

    // Accessors
    public ClassGen getCgen()
    { return (ClassGen)cgen.clone(); }

    // Methods
    public static ClassGen rewriteClass(
            PolicyClass policyClass,
            ClassGen cgen,
            Policy policy,
            HashMap<String, ClassGen> classMap)
        throws Exception
    {
        // This is just a convenience method to avoid having to explicitly
        // create an instance of ClassRewriter prior to rewriting a class
        ClassRewriter classRewriter = new ClassRewriter(
                policyClass, cgen, policy, classMap);
        classRewriter.rewriteClass();
        return classRewriter.getCgen();
    }
    
    public boolean rewriteClass()
        throws Exception
    {
        // Scans through each method and rewrites it as appropriate
        // Returns true if something was rewritten, false otherwise

        // Set up the construction tools
        ConstantPoolGen cpgen = getCgen().getConstantPool();
        String cname = cgen.getClassName();
        Method[] methods = cgen.getMethods();
        Method method;
        MethodGen mgen;
        
        boolean retVal = false;

        // We'll search through every method in the class
        methods = cgen.getMethods();
        for (int i = 0; i < methods.length; i++)
        {
            
            // Prepare the method for modification
            method = methods[i];
            mgen = new MethodGen(method, cname, cpgen);
            
            // Modify the method
            MethodRewriter methodRewriter = new MethodRewriter(
                    policyClass, cgen, mgen, policy, classMap);
            if (methodRewriter.rewriteMethod(method))
                retVal = true;

        }
        
        return retVal;
        
    }

}
